//
// Created by QP on 2025/4/12.
//

#ifndef PCL_WRAPPER_KDTREE_HPP
#define PCL_WRAPPER_KDTREE_HPP
#pragma once
#include <pcl/kdtree/kdtree_flann.h>
#include "cloud_handle.hpp"

namespace pcl_wrapper {

    template<typename PointT>
    class KdTreeWrapper {
    public:
        using PointCloudPtr = typename pcl::PointCloud<PointT>::Ptr;

        KdTreeWrapper() : kdtree(new pcl::KdTreeFLANN<PointT>()) {}

        void setInputCloud(PointCloudPtr cloud) {
            kdtree->setInputCloud(cloud);
        }

        // KNN search
        bool nearestKSearch(const PointT& pt, int k,
                            std::vector<int>& indices, std::vector<float>& distances) {
            indices.resize(k);
            distances.resize(k);
            return kdtree->nearestKSearch(pt, k, indices, distances) > 0;
        }

        // Radius search
        bool radiusSearch(const PointT& pt, double radius,
                          std::vector<int>& indices, std::vector<float>& distances) {
            return kdtree->radiusSearch(pt,radius, indices, distances ) > 0;
        }

    private:
        typename pcl::KdTreeFLANN<PointT>::Ptr kdtree;
    };

    using KdTreeXYZ  = KdTreeWrapper<pcl::PointXYZ>;
    using KdTreeXYZI = KdTreeWrapper<pcl::PointXYZI>;

    // 显式实例化
    template class KdTreeWrapper<pcl::PointXYZ>;
    template class KdTreeWrapper<pcl::PointXYZI>;

}
#endif //PCL_WRAPPER_KDTREE_HPP
